/*
FiltreNum.h - Library for filtering inputs
Version 1.0

by Cordier Yves december 2025 Version du 28/03/2025
release :

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef FiltreNum_h
#define FiltreNum_h

// ***************************

//     Paramêtres additionnels
// *****************************

// Les types d'entrées du filtre complémentaires à INPUT INPUT_PULLUP
// ********************************************
#define _maxResolutionBit (10) // la résolution des valeurs en nombre de bits (10 bits typiquement)
#define _maxIntValInput 2^_maxResolutionBit // Le nombre maxi lors d'une entrée analogique 
// correspond aussi à ce que le système prend pour une entrée TOR

#define INPUT_ANALOG (0x10) // Une entrée analogique

#define INPUT_PTR_BOOL (0x11) // une entrée qui correspond à une variable de type *BOOLEAN
#define INPUT_PTR_CHAR (0x12) // une entrée qui correspond à une variable de tupe *CHAR
#define INPUT_PTR_INT (0x13) // une entrée qui correspond à une variable de type *INT
#define INPUT_PTR_LONG (0x14) // une entrée qui correspond à une variable de type *LONG
#define INPUT_PTR_FLOAT (0x15) // une entrée qui correspond à une variable de type *FLOAT
#define INPUT_PTR_DOUBLE (0x16) // une entrée qui correspond à une variable de tyupe *DOUBLE

// Les types de filtres créés
#define _FO (0)
#define _FN (1)
#define _FNLOPFO (2)
#define _FNHIPFO (3)
#define _FNLOPSO (4)
#define _FNHIPSO (5)
#define _FNCBSO (6)
#define _FNPBSO (7)
#define _FNTR (8)
#define _FNAMP (9)

// Definition du symbole ptr (pointeur) en fonction des processeurs
#if __SIZEOF_POINTER__==1
#define ptr uint8_t
#pragma message("pointeurs 8 bits")
#endif

#if __SIZEOF_POINTER__==2
#define ptr uint16_t
#pragma message("pointeurs 16 bits")
#endif

#if __SIZEOF_POINTER__==4
#define ptr uint32_t
#pragma message("pointeurs 32 bits")
#endif

#if __SIZEOF_POINTER__==8
#define ptr uint64_t
#pragma message("pointeurs 64 bits")
#endif

// un type util pour pouvoir mélanger les entrées

union InputType // Declare a simple union type
{
  int pin;
  bool * bol_ptr;
  char * ch_ptr;
  int * i_ptr;
  long * l_ptr;
  float * f_ptr;
  double * d_ptr;
};

// La classe mêre de tous les filtres
// **********************************

class FilterObject {
  public: byte type = _FO;
  int actualValue = 0; // conditions de HeaverSide
  

  FilterObject() // Createur de base
  {
    setActual(0); // conditions de heaverside
    type = _FO;
  }

  bool getOutBool() // recupère la valeur en tant que booleen
  {
    return (actualValue > (_maxIntValInput/2));
  };

  int getOutVal() // recupère la valeur de sortie en tant qu'entier
  {
    return actualValue;
  }

  void * getValOutPtr() // recupère l'adresse de la valeur de sortie (pour chainage des filtres)
  {
    return ( & actualValue);
  }

  void compute() {}; // calcul l'évolution du filtre
  void init() // initialisation du filtre

  {
    setActual(0); // condition de HeaverSide

  }

  void setActual(int v) // forcage de la valeur de sortie
  {
    actualValue = v;
  }
  ptr *getAdrOut() // utilse pour chainer les filtres via une valeur de type int
  {
    return & actualValue;
  }
  int getInput() // actualisation des entrés
  {}

};

// Les filtres numériques
// **********************

// Le filtre numerique de base
// ***************************
class FilterNumObject: public FilterObject {
  public: InputType input = InputType {
    .pin = 0
  }; // une entrée composite une broche ou une variable (une pin par defaut)
  int mode = INPUT; // défini le type d'entrée : le mode si une broche le type de variable si variable
  int valIn = 0; // la valeur de l'entrée actualisée par getInput();
  float fReelDt = 0; // la fréquence de référence si besoin
  float QReel = 1; // la qualité du filtre si besoin

  FilterNumObject() // Le constructeur par defaut
  {
    // recupère les valeurs prédéfinien
    type = _FN;
  }

  FilterNumObject(InputType in , byte inMode) // Le constructeur évolué
  {
    input = in;
    mode = inMode;
    type = _FN;

  }

  void init() {
    if (mode < 0x10) pinMode(input.pin & 0x00FF, mode); // configuration de l'entrée avec son numero <255
    setActual(0); // conditions de heaverSide
  }

  int getInput() {

    switch (mode) {
    case INPUT:
    case INPUT_PULLUP:
      if (digitalRead(input.pin)) valIn =_maxIntValInput;
      else valIn = 0;
      break;
    case INPUT_ANALOG:
      #ifndef __AVR__
      setAnlogResolution(_maxResolutionBit); // on bloque le nombre de bits
      #endif
      valIn = analogRead(input.pin);
      break;
    case INPUT_PTR_BOOL:
      if ( * input.bol_ptr) valIn = _maxIntValInput;
      else valIn = 0;
      break;
    case INPUT_PTR_CHAR: // une entrée qui correspond à une variable de type *CHAR
      valIn = int( * input.ch_ptr); // input.ch_ptr contient l'adresse de la variable de type char
      // *input.ch_ptr permet d'obtenir la valeur de la variable (* opérateur de Déréférencement)
      break;
    case INPUT_PTR_INT: // une entrée qui correspond à une variable de type *INT
      valIn = int( * input.i_ptr); // input.i_ptr contient l'adresse de la variable de type int
      // *input.i_ptr permet d'obtenir la valeur de la variable (* opérateur de Déréférencement)
      break;
    case INPUT_PTR_LONG: // une entrée qui correspond à une variable de type *LONG
      valIn = int( * input.l_ptr); // input.l_ptr contient l'adresse de la variable de type long
      // *input.long_ptr permet d'obtenir la valeur de la variable (* opérateur de Déréférencement)
      break;
    case INPUT_PTR_FLOAT: // une entrée qui correspond à une variable de type *FLOAT
      valIn = int( * input.f_ptr); // input.f_ptr contient l'adresse de la variable de type float
      // *input.f_ptr permet d'obtenir la valeur de la variable (* opérateur de Déréférencement)
      break;
    case INPUT_PTR_DOUBLE: // une entrée qui correspond à une variable de type *DOUBLE
      valIn = int( * input.d_ptr); // input.d_ptr contient l'adresse de la variable de type double
      // *input.d_ptr permet d'obtenir la valeur de la variable (* opérateur de Déréférencement)
      break;
    };
  };

  float getFreqRef() {
    return fReelDt;
  }
  float getFreqf1() {
    return fReelDt;
  }
  float getFreqf2() {
    return fReelDt;
  }
  float getQuality() {
    return QReel;
  }

};

// Les filtres du premier ordre
// ****************************

/* suivant la valeur de n0 donnée, on passe sur un calcul classique avec multiplications 
   ou si n0=2^p sur un calcul rapide utilisant les décalages
   */
class FilterNumLowPassFirstOrder: public FilterNumObject {
  public: int n = 1;
  int nm1 = 0;
  float unsn = 1;
  int p2 = 0;
  bool fast = false;

  FilterNumLowPassFirstOrder() {
    FilterNumObject();
    type = _FNLOPFO;

  }

  FilterNumLowPassFirstOrder(InputType in , byte inMode, int n0): FilterNumObject(in, inMode) {
    //input=in;
    //mode = inMode;
    type = _FNLOPFO;
    p2 = int(log(n0) / log(2) + 0.5);
    int n1 = int(pow(2, p2) + 0.5); // +0.5 pour arrondir à l'entier le plus proche
    fast = (n0 == n1); // on est en calcul rapide ?
    if (!fast) {
      n = n0;
      nm1 = n - 1;
      unsn = 1. / n;
    }
    fReelDt = n;

  }

  FilterNumLowPassFirstOrder(InputType in , byte inMode, float dt, float fc): FilterNumObject(in, inMode) { // definition en fréquence de dt fc=1/(2*pi*n0*dt)
    // attention n0 sera calculé à partir de dt et fc il sera arrondis à un entier puis fc sera recalculé C.F. getFreQRef

    type = _FNLOPFO;
    int n0 = int(1 / (2 * M_PI) * fc * dt + 0.5); // entier au plus pres
    p2 = int(log(n0) / log(2) + 0.5);
    int n1 = int(pow(2, p2) + 0.5); // +0.5 pour arrondir à l'entier le plus proche
    fast = (n0 == n1); // on est en calcul rapide ?
    if (!fast) {
      n = n0;
      nm1 = n - 1;
      unsn = 1. / n;
    }
    fReelDt = n;
    type = _FNLOPFO;
  }

  void compute() // calcul de la valeur de sortie du filtre
  {
    if (n > 30) {
      float s = (actualValue * nm1 + valIn) * unsn;
      actualValue = constrain(s, -32768, 32767);
    } else {
      if (fast) {
        // sans multiplications
        long s = long(actualValue) << p2;
        actualValue = (s - actualValue + valIn) >> p2;
      } else {
        long s = actualValue * nm1 + valIn;
        actualValue = int(unsn * s + 0.5);
      }
    }

  }

};

class FilterNumHighPassFirstOrder: public FilterNumLowPassFirstOrder {

  public: int actualValueLowPass = 0;

  FilterNumHighPassFirstOrder(InputType in , byte inMode, int n0) // definition en fréquence de dt fc=1/(2*pi*n0*dt)
  : FilterNumLowPassFirstOrder(in, inMode, n0) {
    actualValueLowPass = 0;
    type = _FNHIPFO;
  }
  FilterNumHighPassFirstOrder(InputType in , byte inMode, float dt, float fc) // definition en fréquence de dt fc=1/(2*pi*n0*dt)
  // attention n0 sera calculé à partir de dt et fc il sera arrondis à un entier puis fc sera recalculé C.F. getFreQRef
  : FilterNumLowPassFirstOrder(in, inMode, dt, fc) {
    actualValueLowPass = 0;
    type = _FNHIPFO;
  }

  void compute() // calcul de la valeur de sortie du filtre
  {
    actualValue = actualValueLowPass;
    FilterNumLowPassFirstOrder::compute();
    actualValueLowPass = actualValue;
    actualValue = FilterNumLowPassFirstOrder::valIn - actualValueLowPass;
  }

  void init() {
    FilterNumLowPassFirstOrder::init();
    actualValueLowPass = 0;
  }

};

// Les filtres du deuxieme ordre
// ****************************

class FilterNumLowPassSecondOrder: public FilterNumObject {
  public: int n = 1;
  int m = 1;
  int sm1 = 0;
  int sm2 = 0;
  float a0 = 0;
  float a1 = 0;
  float b0 = 0;

  FilterNumLowPassSecondOrder() {
    FilterNumObject();
    type = _FNLOPSO;
  }

  FilterNumLowPassSecondOrder(InputType in , byte inMode, int n0, int m0) // constructeur avec la définition en n0, m0 Q=m0/n0  f0=1/(2*pi*m0*dt)
  : FilterNumObject(in, inMode) {
    type = _FNLOPSO;
    n = n0;
    m = m0;
    fReelDt = 1 / (2 * M_PI * n);
    QReel = float(m) / n;
    a0 = (n + 2. * m * m - 1) / (n + m * m);
    a1 = -float(m * m) / (n + m * m);
    b0 = 1. / (n + m * m);
  }

  FilterNumLowPassSecondOrder(InputType in , byte inMode, float dt, float fc, float Q) // definition en fréquence de dt fc=1/(2*pi*n0*dt) et qualité
  : FilterNumObject(in, inMode) {
    // n et m seront calculés en fonction de fc et Q (arrondis à l'entier le plus proche) puis la valeur réelle de fc et Q seront recalculés
    // C.F. getFreqRef et getQuality
    type = _FNLOPSO;
    n = max(int(1 / (2 * M_PI * fc * dt) + 0.5), 1);
    m = int(Q * n + 0.5);
    fReelDt = 1 / (2 * M_PI * n);
    QReel = float(m) / n;
    a0 = (n + 2. * m * m - 1) / (n + m * m);
    a1 = -float(m * m) / (n + m * m);
    b0 = 1. / (n + m * m);
  }

  void compute() // calcul de la valeur de sortie du filtre
  {
    actualValue = constrain(int(a0 * sm1 + a1 * sm2 + b0 * valIn + 0.5), -32768., 32767.);
    sm2 = sm1;
    sm1 = actualValue;
  }
  float getQreel() {return QReel;} // renvoi la valeur calculée de Q

};

class FilterNumHighPassSecondOrder: public FilterNumLowPassSecondOrder {

  public: int actualValueLowPass = 0;

  FilterNumHighPassSecondOrder(InputType in , byte inMode, int n0, int m0) // constructeur avec la définition en n0, m0 Q=n0/m0  f0=1/(2*pi*m0*dt)

  : FilterNumLowPassSecondOrder::FilterNumLowPassSecondOrder(in, inMode, n0, m0) {
    actualValueLowPass = 0;
    type = _FNHIPSO;
  }
  FilterNumHighPassSecondOrder(InputType in , byte inMode, float dt, float f0, float Q) // definition en fréquence de dt fc=1/(2*pi*n0*dt) et qualité
  // n et m seront calculés en fonction de fc et Q (arrondis à l'entier le plus proche) puis la valeur réelle de fc et Q seront recalculés
  // C.F. getFreqRef et getQuality

  : FilterNumLowPassSecondOrder::FilterNumLowPassSecondOrder(in, inMode, dt, f0, Q) {
    actualValueLowPass = 0;
    type = _FNHIPSO;
  }

  void compute() // calcul de la valeur de sortie du filtre

  {
    actualValue = actualValueLowPass;
    FilterNumLowPassSecondOrder::compute();
    actualValueLowPass = actualValue;
    actualValue = FilterNumLowPassSecondOrder::valIn - actualValueLowPass;
  }

};
// Les filtres du deuxieme ordre coupe et passe bande
// **************************************************
class FilterNumCutBandSecondOrder: public FilterNumObject {
  public: int n = 1;
  int m = 1;
  int sm1 = 0;
  int sm2 = 0;
  int em1 = 0;
  int em2 = 0;
  float a0 = 0;
  float a1 = 0;
  float b0 = 0;
  float b1 = 0;
  float b2 = 0;
  float f2ReelDt = 0;
  float f1ReelDt = 0;

  FilterNumCutBandSecondOrder() {
    FilterNumObject();
    type = _FNCBSO;
  }

  FilterNumCutBandSecondOrder(InputType in , byte inMode, int n0, int m0) // constructeur avec la définition en n0, m0 Q=n0/m0  f0=1/(2*pi*m0*dt)
  : FilterNumObject(in, inMode) {
    type = _FNCBSO;
    n = n0;
    m = m0;
    calculCoefs();
  }

  FilterNumCutBandSecondOrder(InputType in , byte inMode, float dt, float fc, float Q) // definition en fréquence de dt fc=1/(2*pi*n0*dt) et qualité
  // n et m seront calculés en fonction de fc et Q (arrondis à l'entier le plus proche) puis la valeur réelle de fc, fc1, fc2et Q seront recalculés 
  // C.F. getFreqRef getFreqf1 getFreqf2  et getQuality
  : FilterNumObject(in, inMode) {
    type = _FNCBSO;
    m = max(int(1 / (2 * M_PI * fc * dt) + 0.5), 1);
    n = max(int(Q * m + 0.5),1);
    calculCoefs();
	}

  FilterNumCutBandSecondOrder(InputType in , byte inMode, float dt, float *f1, float *f2) // definition en bande de fréquence f1 <f2
  // Les paramètres d'entrée ont été mis en tant que pointeurs pour que le compilateur fasse la différence avec le constructeur 
  // précédent en frequence et qualité
  // On recalcul fc et Q
  // f1=fc*1/Q f2=fc*(Q+1/Q) => fc=(fi*f2-f1^2)^0.5; n=1/(2*pi*fc*dt); Q=fc/f1; Q=m/n => m=Q*n
  // n et m seront calculés en fonction de fc et Q (arrondis à l'entier le plus proche) puis la valeur réelle de fc et Q seront recalculés
  // idem pour la définition en f1 f2
  // C.F. getFreqRef , getQuality, getFreqf1 et getFreqf2
  : FilterNumObject(in, inMode) {
    type = _FNCBSO;
    float fc = sqrt(*f2 * *f1);
    m = max(int(1 / (2 * M_PI * fc * dt) + 0.5), 1);
	QReel = fc/(*f1-*f2);
    fc = 1 / (2 * M_PI * m*dt);
    QReel = fc/(*f1-*f2);
    m = int(QReel * n + 0.5);
    calculCoefs();
	}
  void calculCoefs()
  { fReelDt = 1 / (2 * M_PI * m);
    QReel = float(n) / m;
    f1ReelDt = fReelDt *(sqrt(4*QReel*QReel+1)+1)/(2*QReel);
    f2ReelDt = fReelDt *(sqrt(4*QReel*QReel+1)-1)/(2*QReel);
    b2 = 1. / (1. / (m * m) + 1 / n + 1);
    b1 = -2 * b1;
    b0 = (1. / (m * m) + 1) * b2;
    a0 = (1. / n + 1) * b2;
    a1 = -b2;
    b0 = 1. / (n + m * m);
	type = _FNCBSO;
  }
	  
  void compute() // calcul de la valeur de sortie du filtre
  {
    actualValue = a0 * sm1 + a1 * sm2 + b0 * valIn + b1 * em1 + b2 * em2;
    sm2 = sm1;
    sm1 = actualValue;
    em2 = em1;
    em1 = valIn;
  }

  float getFreqf1() {
    return f1ReelDt;
  }
  float getFreqf2() {
    return f2ReelDt;
  }
  float getQreel() {return QReel;} // renvoi la valeur calculée de Q
};

class FilterNumPassBandSecondOrder: public FilterNumCutBandSecondOrder {

  public: int actualValueLowPass = 0;

  FilterNumPassBandSecondOrder(InputType in , byte inMode, int n0, int m0) // constructeur avec la définition en n0, m0 Q=n0/m0  f0=1/(2*pi*m0*dt)

  : FilterNumCutBandSecondOrder::FilterNumCutBandSecondOrder(in, inMode, n0, m0) {
    actualValueLowPass = 0;
    type = _FNPBSO;
  }
  FilterNumPassBandSecondOrder(InputType in , byte inMode, float dt, float f0, float Q0) // definition en fréquence de dt fc=1/(2*pi*n0*dt) et qualité
  // n et m seront calculés en fonction de fc et Q (arrondis à l'entier le plus proche) puis la valeur réelle de fc, fc1, fc2et Q seront recalculés 
  // C.F. getFreqRef getFreqf1 getFreqf2  et getQuality

  : FilterNumCutBandSecondOrder::FilterNumCutBandSecondOrder(in, inMode, dt, f0, Q0) {
    actualValueLowPass = 0;
    type = _FNPBSO;
  }
  FilterNumPassBandSecondOrder(InputType in , byte inMode, double dt, float *f1, float *f2) // definition en bande de fréquence f1 <f2
  // On recalcul fc et Q
  // f1=fc*1/Q f2=fc*(Q+1/Q) => fc=(fi*f2-f1^2)^0.5; n=1/(2*pi*fc*dt); Q=fc/f1; Q=m/n => m=Q*n
  // n et m seront calculés en fonction de fc et Q (arrondis à l'entier le plus proche) puis la valeur réelle de fc et Q seront recalculés
  // idem pour la définition en f1 f2
  // C.F. getFreqRef , getQuality, getFreqf1 et getFreqf2

  : FilterNumCutBandSecondOrder::FilterNumCutBandSecondOrder(in, inMode, dt, *f1, *f2) {
    actualValueLowPass = 0;
    type = _FNPBSO;
  }

  void compute() // calcul de la valeur de sortie du filtre
  {
    int sto = actualValue;
    actualValue = actualValueLowPass;
    FilterNumCutBandSecondOrder::compute();
    actualValueLowPass = actualValue;
    actualValue = FilterNumCutBandSecondOrder::valIn - actualValueLowPass;
  }

};

// Les filtres particuliers
// ***********************

// Amplification du signal

class FilterNumAmplif: public FilterNumObject {
  public: float amplif = 1;
  FilterNumAmplif(InputType in , byte inMode, float v) // constructeur avec facteur amplification
  : FilterNumObject(in, inMode) {
    input = in;
    mode = inMode;
    amplif = v;
    type = _FNAMP;
  }
  void compute() // calcul de la valeur de sortie du filtre
  {
    actualValue = int(valIn * amplif + 0.5);
  }

};

// trigger de schmidt avec ou sans inversion du signal
// inversion si satb>sath dans le createur

class FilterNumTrigger: public FilterNumObject {
  public: int dec = 0;
  bool sens = true;
  bool out = false;
  int seuilBas=512;
  int seuilHaut=512;
  int saturationHaute=1023;
  int saturationBasse=0;
  FilterNumTrigger(InputType in , byte inMode, int sb, int sh, int satb, int sath) // constructeur avec seuil bas, seuil haut saturation basse et saturation haute
  : FilterNumObject(in, inMode) {
    input = in;
    mode = inMode;
    if (sb<=sh) 
	  {seuilBas=sb;
	  seuilHaut=sh;}
	else
	  {seuilBas=sh;
	  seuilHaut=sb;}	
	saturationHaute=sath;
	saturationBasse=satb;
	type = _FNTR;
  }
  
  void compute() // calcul de la valeur de sortie du filtre
  {
    if (sens) // on est en phase d'augmentation de signal
    {
      if (valIn > seuilHaut) {
        out = true;
        sens = false;
      } else out = false;
    } else // on est en phase de descente de signal
    {
      if (valIn <= seuilBas) {
        out = false;
        sens = true;
      } else out = true;
    }
  };

  int getOutVal() // recupère la valeur de sortie en tant qu'entier
  {
    if (out) return saturationHaute;
    else return saturationBasse;
  }
  bool getOutBool() // recupère la valeur booleenne de sortie
  {
    return out;
  }

};

// Les structures pour faire fonctionner ces filtres
// *************************************************

// La structure chainée de tous les filtres synchrones
union contents {
  FilterObject * fo;
  FilterNumObject * fn;
  FilterNumLowPassFirstOrder * fnlopfo;
  FilterNumHighPassFirstOrder * fnhipfo;
  FilterNumLowPassSecondOrder * fnlopso;
  FilterNumHighPassSecondOrder * fnhipso;
  FilterNumCutBandSecondOrder * fncbso;
  FilterNumPassBandSecondOrder * fnpbso;
  FilterNumAmplif * fnamp;
  FilterNumTrigger * fntr;
};

struct filter {
  filter * next;
  contents cont;
};

// La variable qui sert à actionner tous les filtres
filter * stackFilter = NULL;

// fonction d'ajout d'entrées filtrées
void addFilter(FilterObject * f) {

  if (stackFilter == NULL) {
    stackFilter = new filter;
    stackFilter -> cont.fo = f;
    stackFilter -> next = NULL;
  } else {
    filter * fil = stackFilter;
    while (fil -> next != NULL) fil = fil -> next;
    fil -> next = new filter;
    fil = fil -> next;
    fil -> cont.fo = f;
    fil -> next = NULL;
  }
}

// Le processus d'interruption et timer
// ************************************

// La procedure de prise des entrées
void takeInputs() {
  filter * f = stackFilter;
  while (f != NULL) {
    //Serial.print(" type :"); Serial.print(f->cont.fo->type); Serial.print(" ");
    switch (f -> cont.fo -> type) {
    case _FO: 		f -> cont.fo -> getInput(); break;
    case _FN : 		f->cont.fn->getInput(); break;
	case _FNLOPFO : f->cont.fnlopfo->getInput(); break;
	case _FNHIPFO : f->cont.fnhipfo->getInput(); break;
	case _FNLOPSO : f->cont.fnlopso->getInput(); break;
	case _FNHIPSO : f->cont.fnhipso->getInput(); break;
	case _FNCBSO : 	f->cont.fncbso->getInput(); break;
	case _FNPBSO :	f->cont.fnpbso->getInput(); break;
	case _FNAMP : 	f->cont.fnamp->getInput(); break;
	case _FNTR:  	f->cont.fntr->getInput(); break;
    }
    f = f -> next;
  }
  //Serial.println();
}
// la procédure de traitement des filtres
void treatmentFilters() {
  filter * f = stackFilter;
  while (f != NULL) {
    switch (f -> cont.fo -> type) {
    case _FO:
      f -> cont.fo -> compute();
      break;
    case _FN:
      f -> cont.fn -> compute();
      break;
    case _FNLOPFO:
      f -> cont.fnlopfo -> compute();
      break;
    case _FNHIPFO:
      f -> cont.fnhipfo -> compute();
      break;
    case _FNLOPSO:
      f -> cont.fnlopso -> compute();
      break;
    case _FNHIPSO:
      f -> cont.fnhipso -> compute();
      break;
    case _FNCBSO:
      f -> cont.fncbso -> compute();
      break;
    case _FNPBSO:
      f -> cont.fnpbso -> compute();
      break;
    case _FNAMP:
      f -> cont.fnamp -> compute();
      break;
    case _FNTR:
      f -> cont.fntr -> compute();
      break;
    }

    f = f -> next;
  }
}

// initialisation de la pile de filtres
void initFilters() {
  filter * f = stackFilter;
  while (f != NULL) {
    switch (f -> cont.fo -> type) {
    case _FO:
      f -> cont.fo -> init();
      break;
    case _FN:
      f -> cont.fn -> init();
      break;
    case _FNLOPFO:
      f -> cont.fnlopfo -> init();
      break;
    case _FNHIPFO:
      f -> cont.fnhipfo -> init();
      break;
    case _FNLOPSO:
      f -> cont.fnlopso -> init();
      break;
    case _FNHIPSO:
      f -> cont.fnhipso -> init();
      break;
    case _FNCBSO:
      f -> cont.fncbso -> init();
      break;
    case _FNPBSO:
      f -> cont.fnpbso -> init();
      break;
    case _FNAMP:
      f -> cont.fnamp -> init();
      break;
    case _FNTR:
      f -> cont.fntr -> init();
      break;
    }

    f = f -> next;
  }
}
// La procedure d'interruption à greffer au timer
// typiquement avec la bibliothèque timerone
// mise en place du timer avec sa valeur de selection
//  Timer1.initialize(dt); // mise en place de la procédure d'interrruption (effedtuée tout les dt
//  Timer1.attachInterrupt(treatmentFilters, 1000); // attache la procedure treatmentFilters au timer
void interTimer() {

  takeInputs();

  treatmentFilters();
}
#endif